% Demo_ASDoppler
% Dmo analyse spectrale du signal Doppler
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

clear
load doppler

fe = 5000;		% Frquence d'chantillonnage
nbsig = 1;		% Numro du signal trait
N = 20000;		% Nombre d'chantillons significatifs
n = 50;			% longueur des fentres d'analyse
K = 64;			% Nombre d'chantillons pour le calcul des TF
x = doppler(1:N, nbsig);

% nu = (0.5-1/K:-1/K:-0.5)';
nu = (-0.5:1/K:0.5-1/K)';
t = (0:n/fe:(N-n)/fe);
Xr = reshape(x, n, N/n);
X = (abs(fft(Xr, K)).^2) / n;
X = [X(K/2+1:K, :); X(1:K/2, :)];
X(find(X==0)) = min(X(find(X~=0)))*ones(size(find(X==0)));
				% Pour viter le log de 0
                
% lambda = 1e04;				% Paramtre de rgularisation
lambda = 1e02;				% Paramtre de rgularisation

Titre = 'Analyse spectrale du signal Doppler';
Item2 = 'Estimer le spectre';
Item3 = 'Quitter';
TODO = 0;

while TODO ~= 3
    
    Item1 = sprintf('Paramtre de rgularisation : %3.1e ', lambda);
    TODO = menu(Titre, Item1, Item2, Item3);
    
    switch TODO
        
        case 1
            lambda_n = ...
                 input(sprintf(['Paramtre de rgularisation ', ...
                 '[Dfaut %3.1e] : '], lambda));
            
            if ~ isempty(lambda_n)
               lambda = lambda_n;
            end
             
        case 2
            a = zeros(n-1, N/n);	% Vecteurs de prdiction sur chaque bloc
            sigma2 = zeros(1, N/n); % Variance des residus
            
            h_w = waitbar(0,'Traitement des donnes Doppler...');
            for nb = 1:N/n			% Sur chaque bloc
                waitbar(nb*n/N, h_w)
                M = toeplitz(Xr(1:n-1, nb), [Xr(1, nb), zeros(1, n-2)]);
                a(:, nb) = inv(M'*M + lambda*eye(n-1)) * (M' * Xr(2:n, nb));
                sigma2(nb) = sum(abs(M * a(:, nb) - Xr(2:n, nb)).^2) / (n-1);
            end
            close(h_w)
            S = sigma2(ones(K, 1), :) ./ (abs(fft([ones(1, N/n);-a], K)).^2);
            S = [S(K/2+1:K, :); S(1:K/2, :)];
            
            figure(1), clf
            set(gcf, 'WindowStyle', 'Docked')
            
            imagesc(t,nu, log(X)), colormap gray, axis xy
            title('PRIODOGRAMME', 'FontSize', 14)
            xlabel('Temps (s.)', 'FontSize', 12)
            ylabel('Frquence rduite', 'FontSize', 12)

            figure(2)
            set(gcf, 'WindowStyle', 'Docked')

            imagesc(t, nu, log(S)), colormap gray, axis xy
            title('AR LONGS', 'FontSize', 14)
            xlabel('Temps (s.)', 'FontSize', 12)
            ylabel('Frquence rduite', 'FontSize', 12)

    end
    
end
