% Dmonstration, analyse spectrale paramtrique
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

clear

N = 1024;
Wn = .3;
ordre = 2;

[B, A] = butter(ordre, Wn);

RF = abs(fftshift(fft(B, N) ./ fft(A, N)));
nu = fftshift((0:N-1)/N);
nu(find(nu > 1/2)) = nu(find(nu > 1/2)) - 1;

figure(3), clf
set(gcf, 'WindowStyle', 'Docked')

semilogy(nu, RF.^2), grid
set(gca, 'Ylim', [1e-04, 1e+01])
title('SPECTRE EXACT DU SIGNAL', 'Fontsize', 14)
xlabel('Frquences rduites', 'Fontsize', 12)

% Analyse spectrale paramtrique

Lsig = 6;
p = 32;
lambda = 1e-2;

Titre = 'Analyse spectrale paramtrique';
Item4 = 'Estimer le spectre';
Item5 = 'Quitter';
TODO = 0;

while TODO ~= 5
    
    Item1 = sprintf('Longueur du signal : %1.0f (2^%1.0f) ', 2^Lsig, Lsig);
    Item2 = sprintf('Ordre du modle : %1.0f ', p);
    Item3 = sprintf('Paramtre de rgularisation : %3.1e ', lambda);

    TODO = menu(Titre, Item1, Item2, Item3, Item4, Item5);
    
    switch TODO
        
        case 1
            Lsig_n = ...
                input(sprintf(['Longueur du signal en puissance de 2 ', ...
                '[Dfaut 2^%1.0f = %1.0f] : '], Lsig, 2^Lsig));
            
            if ~ isempty(Lsig_n)
                Lsig = Lsig_n;
            end
            
        case 2
            p_n = ...
                input(sprintf(['Ordre du modle ', ...
                '[Dfaut %1.0f] : '], p));
            
            if ~ isempty(p_n)
                p = p_n;
            end
            
        case 3
            lambda_n = ...
                input(sprintf(['Paramtre de rgularisation ', ...
                '[Dfaut %3.1e] : '], lambda));
            
            if ~ isempty(lambda_n)
                lambda = lambda_n;
            end
            
            
        case 4            
            randn('seed', 0);
            u = randn(2^Lsig, 1);

            x = filter(B, A, u);

            figure(3), clf

            plot(1:2^Lsig, x), grid
            xlabel('Nombre d''chantillons', 'Fontsize', 12)
            ylabel('Amplitude', 'Fontsize', 12)
            title('SIGNAL  ANYALSER', 'Fontsize', 14)

            rho = 0.999;
            regul = toeplitz(rho.^(0:p-1));
            
            M = toeplitz(x(1:2^Lsig-1), [x(1), zeros(1, p-1)]);
            a = inv(M'*M + lambda*eye(p)) * (M' * x(2:2^Lsig));
%            a = inv(M'*M + lambda*regul) * (M' * x(2:2^Lsig));
            sigma2 =  sum(abs(M * a - x(2:2^Lsig)).^2) / (2^Lsig-1);
            Sp_X = sigma2 ./ abs(fft([1; -a], 2^Lsig)).^2;
            nu_X = -1/2:2^-Lsig:(1/2-2^-Lsig);
            
            titre = sprintf('Spectre estim, ordre %1.0f, lambda = %3.1e', ...
                p, lambda);
  
            figure(4), clf
            set(gcf, 'WindowStyle', 'Docked')

            semilogy(nu_X, fftshift(Sp_X)), hold on
            semilogy(nu, RF.^2, 'r'), hold off, grid
            xlabel('Frquences rduites', 'Fontsize', 12)

            title(upper(titre), 'Fontsize', 14)
            set(gca, 'Ylim', [1e-04, 1e+01])
            
    end
    
end

